@@ -21,12 +21,13 @@ our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 sub _CERTIFICATE_HEADER_SIZE { return 40; }
 sub _MAX_PUBLIC_KEY_SIZE     { return 65_536; }
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 sub pin_sha256 {
     my ($path) = @_;
     my $handle = FileHandle->new($path)
       or Carp::croak("Failed to open $path for reading:$EXTENDED_OS_ERROR");
+    binmode $handle;
     read $handle, my $file_header, _CERTIFICATE_HEADER_SIZE()
       or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
     my $pem_encoded_public_key_string;
@@ -34,7 +35,7 @@ sub pin_sha256 {
         /^[-]{5}BEGIN[ ](?:X[.]?509[ ]|TRUSTED[ ])?CERTIFICATE[-]{5}/smx )
     {
         $pem_encoded_public_key_string =
-          _process_pem_certificate( $handle, $file_header, $path );
+          _process_pem_x509_certificate( $handle, $file_header, $path );
     }
     elsif ( $file_header =~ /^[-]{5}BEGIN[ ](?:RSA[ ])?PUBLIC[ ]KEY[-]{5}/smx )
     {
@@ -43,7 +44,9 @@ sub pin_sha256 {
     }
     else {
         $pem_encoded_public_key_string =
-          _check_for_der_certificate( $handle, $file_header, $path );
+          _check_for_der_encoded_x509_certificate( $handle, $file_header,
+            $path )
+          || _check_for_der_encoded_public_key( $handle, $file_header, $path );
         if ( !defined $pem_encoded_public_key_string ) {
             Carp::croak("$path is not an X.509 Certificate");
         }
@@ -62,7 +65,7 @@ sub pin_sha256 {
     return $base64;
 }
 
-sub _process_pem_certificate {
+sub _process_pem_x509_certificate {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
     if ( $file_header =~ /^[-]{5}BEGIN[ ]CERTIFICATE[-]{5}/smx ) {
@@ -90,7 +93,7 @@ s/^([-]{5}BEGIN[ ])(?:X[.]?509|TRUSTED)[ ](CERTIFICATE[-]{5})/$1$2/smx;
     return $pem_encoded_public_key_string;
 }
 
-sub _check_for_der_certificate {
+sub _check_for_der_encoded_x509_certificate {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
     eval {
@@ -104,6 +107,38 @@ sub _check_for_der_certificate {
     return $pem_encoded_public_key_string;
 }
 
+sub _check_for_der_encoded_public_key {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    seek $handle, 0, Fcntl::SEEK_SET()
+      or Carp::croak("Failed to seek to start of $path:$EXTENDED_OS_ERROR");
+    defined read $handle, my $der_encoded_public_key_string,
+      _MAX_PUBLIC_KEY_SIZE()
+      or Carp::croak("Failed to read from $path:$EXTENDED_OS_ERROR");
+    my $asn = Convert::ASN1->new( encoding => 'DER' );
+    $asn->prepare(
+        <<"__SUBJECT_PUBLIC_KEY_INFO__") or Carp::croak( 'Failed to prepare SubjectPublicKeyInfo in ASN1:' . $asn->error() );
+SubjectPublicKeyInfo ::= SEQUENCE {
+  algorithm SEQUENCE { algorithm OBJECT IDENTIFIER, parameters ANY OPTIONAL },
+  subjectPublicKey BIT STRING
+}
+__SUBJECT_PUBLIC_KEY_INFO__
+    eval {
+        my $pub_key = $asn->decode($der_encoded_public_key_string)
+          or Carp::croak(
+            'Failed to decode SubjectPublicKeyInfo in ASN1:' . $asn->error() );
+        if ( $pub_key->{subjectPublicKey} ) {
+            $pem_encoded_public_key_string =
+                "-----BEGIN PUBLIC KEY-----\n"
+              . MIME::Base64::encode_base64($der_encoded_public_key_string)
+              . "-----END PUBLIC KEY-----\n";
+        }
+    } or do {
+        return;
+    };
+    return $pem_encoded_public_key_string;
+}
+
 sub _process_pem_public_key {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
@@ -149,7 +184,7 @@ HTTP::PublicKeyPins - Generate RFC 7469 HTTP Public Key Pin (HPKP) header values
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =head1 SYNOPSIS
 
@@ -176,7 +211,7 @@ This module allows the calculation of RFC 7469 HTTP Public Key Pin header values
 
 =head2 pin_sha256
 
-This function accepts the path to a L<X.509 Certificate|https://en.wikipedia.org/wiki/X.509>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM format).  Patches for other formats are encouraged.
+This function accepts the path to a L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM (L<SubjectPublicKeyInfo|http://tools.ietf.org/html/rfc5280#section-4.1.2.7> or L(PKCS#1|https://tools.ietf.org/html/rfc344) or DER format).
 
 =head1 SUBROUTINES/METHODS
 
@@ -188,18 +223,22 @@ None.  This module only has the one exported function.
  
 =item C<< Failed to open %s for reading >>
  
-Failed to open the supplied X.509 Certificate file
+Failed to open the supplied X.509 Certificate or Public Key file
  
 =item C<< Failed to read from %s >>
  
-Failed to read from the supplied X.509 Certificate file
+Failed to read from the X.509 Certificate or Public Key file
 
-=item C<< %s is not an X.509 Certificate >>
+=item C<< %s is not an X.509 Certificate or Public Key >>
  
-The supplied input file does not look like X.509 Certificate File. The X.509 Certificate file may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
+The supplied input file does not look like X.509 Certificate File or Public Key. The X.509 Certificate file may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
 
   -----BEGIN CERTIFICATE-----
 
+A PEM encoded Public Key has the following header
+
+  -----BEGIN PUBLIC KEY-----
+
 =back
  
 =head1 CONFIGURATION AND ENVIRONMENT
@@ -224,7 +263,9 @@ None known.
 
 =item L<RFC 7469 - Public Key Pinning Extension for HTTP|http://tools.ietf.org/html/rfc7469>
 
-=item L<X.509 Certificate|https://en.wikipedia.org/wiki/X.509>
+=item L<X.509 Certificate|http://tools.ietf.org/html/rfc5280>
+
+=item L<PKCS#1|https://tools.ietf.org/html/rfc344>
 
 =back
 
